function coding_demo

% Demonstration of DPCM and DCT image coding
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Call coding_demo

m=menu('Select','Prediction errors','DPCM-test','DCT-test');
m0=menu('Select image','Chest X-ray','Angiogram','Text');
switch m0,
    case 1, load chest;INPIMG=chest;
    case 2, load ango;INPIMG=ango;
    case 3, load MRI;INPIMG=MRI;
end
switch m,
case 1,
   dpcm_test(INPIMG);
case 2,
   m2=menu('Select number of BPP','2','3');
%    chest=chest(1:2:end,1:2:end);
   OUTIMG=dpcm_1D(INPIMG,50,m2+1,0.3);
case 3,
   m31=menu('Select the size of the coding mask devider','4','5','6','7','8');
   if m0==1,INPIMG=chest(1:2:end,1:2:end);end
   m32=menu('Select number of bit per sample','2','3','4','5');
   OUTIMG=dct_codng(INPIMG,8,m31+3,m32+1,0.3);
end

%--------------------------------------------------------------------------
function dpcm_test(INPIMG);
% Horizontal, vertical and 2-D prediction for DPCM:
% evaluation of variances and images
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Call dpcm_test(INPIMG)

fig_n(4);

[mn, v]=std_2d(INPIMG);
figure(1);
subplot(121);
display_1(INPIMG);title(['Input image; std=',num2str(v)],'FontWeight','Bold');drawnow
imh=img_histo(INPIMG);
subplot(122);
bar(imh);
title('Input image histogram','FontWeight','Bold');
xlabel('Gray level','FontWeight','Bold')
grid on;drawnow
set(gca,'FontWeight','Bold');

maskhor=[0 0 0;-1 1 0;0 0 0];
maskver=[0 -1 0;0 1 0;0 0 0];
mask2d= [-0.2 -0.3 -0.2; -0.3 1 0; 0 0 0];

hordiff=conv2(INPIMG,maskhor,'same');
[mn,stdhordiff]=std_2d(hordiff);
figure(2);colormap(gray(256))
subplot(121);disp_norm(hordiff,4);
title(['Horizontal differences; std=',num2str(stdhordiff)],'FontWeight','Bold');
drawnow
xlabel('Abs(difference)','FontWeight','Bold')
absdif=abs(hordiff);mx=max(max(absdif));
absdif=round(255*absdif/mx);
hdiff=img_histo(absdif);
subplot(122);
bar(hdiff/max(hdiff),'r');grid
axis([0 50 0 1]);
title('Histogram of horizontal differences','FontWeight','Bold');drawnow
xlabel('Abs(difference)','FontWeight','Bold')
set(gca,'FontWeight','Bold');
verdiff=conv2(INPIMG,maskver,'same');
[mn,stdverdiff]=std_2d(verdiff);
figure(3);colormap(gray(256))
subplot(121);disp_norm(verdiff,4);
title(['Vertical differences; std=',num2str(stdverdiff)],'FontWeight','Bold');drawnow
set(gca,'FontWeight','Bold');
absdif=abs(verdiff);mx=max(max(absdif));
absdif=round(255*absdif/mx);
hdiff=img_histo(absdif);
subplot(122);
bar(hdiff/max(hdiff),'g');axis([0 50 0 1]);grid
title('Histogram of vertical differences','FontWeight','Bold');drawnow
xlabel('Abs(difference)','FontWeight','Bold')
set(gca,'FontWeight','Bold');
horverdiff=conv2(INPIMG,mask2d,'same');
[mn,stdhorverdiff]=std_2d(horverdiff);
figure(4);colormap(gray(256))
subplot(121);disp_norm(horverdiff,4);
title(['2D prediction error; std=',num2str(stdhorverdiff)],'FontWeight','Bold');drawnow
set(gca,'FontWeight','Bold');
absdif=abs(horverdiff);mx=max(max(absdif));
absdif=round(255*absdif/mx);
hdiff=img_histo(absdif);
subplot(122);
bar(hdiff/max(hdiff),'b');grid
axis([0 50 0 1]);
title('Histogram of 2-D prediction error','FontWeight','Bold');
xlabel('Abs(difference)','FontWeight','Bold')
set(gca,'FontWeight','Bold');
drawnow

%-----------------------------------------------------------------
function fig_n(N)
% Generating N square figure windows of ScreenVerSz/2*ScreenVerSz pixels
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Version July 15, 2001
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call fig_n(N)

% Generating a figure window
ScrSz=get(0,'ScreenSize');
HorSz=(ScrSz(3)-20); VerSz=(ScrSz(4));
close all;

switch N;
case 1, 
   Nvert=1;Nhor=1;H_Sz=(HorSz-80); V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-V_Sz V_Sz V_Sz]); colormap(gray(256))
case 2, 
   Nvert=1;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
case 3, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 4, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 5,
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 6, 
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-75-2*V_Sz H_Sz V_Sz]);colormap(gray(256))
end
%------------------------------------------------------------------------------

function display_1(INPIMG)
% Displays a 2-D signal as an image 
% with normalization by gobal maximum and minimum
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Call display_1(INPIMG)

colormap(gray(256))
Min=min(min(INPIMG)); Max=max(max(INPIMG));
if Max-Min==0, im=zeros(size(INPIMG)); else im=255*(INPIMG-Min)/(Max-Min); end

image(im);axis image;axis off;
%-------------------------------------------------------------------------------

function disp_norm(INPIMG,g)
% 2-D signal display with normalization by mean value and standard deviation
% g- is a parameter that defines ratio of the display range 0-255
% to the signal standard deviation 
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Call disp_norm(INPIMG,g)

[SzX SzY]=size(INPIMG);
Mean=mean(mean(INPIMG));		
stdv=sqrt(mean(mean((INPIMG-Mean).^2)))+eps;
dspl=round(255*(INPIMG-Mean)/(g*stdv))+128;
dspl=(dspl>=0).*dspl;
dspl=(dspl<=255).*dspl+255*ones(SzX,SzY).*(dspl>255);
image(dspl);axis image; axis off
%---------------------------------------------------------------------------
function [mn,v]=std_2d(INP);

% Standard deviation and mean of a 2-D array
% z=[mean stdeviation];
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call [mn v]=std_2d(INP)

[SzX SzY]=size(INP);
mn=sum(sum(INP))/(SzX*SzY);
v=sqrt(sum(sum((INP-mn).^2))/(SzX*SzY));

%----------------------------------------------------------------------------

function OUTIMG=dpcm_1D(INPIMG,mxm,BPP,P);
% Simulation of DPCM with 1D prediction;
% Difference signal quantization on 2^(BBP-1) levels in the range [-mxm,mxm]
% with nonlinear (P)-law predistorion;
% Additive or impulse noise in the difference errorr transmission channel;
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Call : OUTIMG=dpcm_1D(INPIMG,mxm,BPP,P);

Q=2^(BPP-1);
[SzX SzY]=size(INPIMG);
fig_n(3);

figure(1);colormap(gray(256));
image(INPIMG);axis off;axis equal;
title('DPCM-1D: Input image','FontWeight','Bold');drawnow

OUTIMG=zeros(SzX,SzY);OUTIMG(:,1)=INPIMG(:,1);

figure(2);colormap(gray(256));
for y=2:SzY,
	diff=INPIMG(:,y)-OUTIMG(:,y-1);
	signdiff=sign(diff);
	absdiff=abs(diff);
    absdiff=absdiff.*(absdiff<=mxm)+mxm*(absdiff>mxm);
    absdiff_quant=round(Q*((absdiff/mxm).^P));
	diff_quant=signdiff.*absdiff_quant;
    diff_quant_noise=diff_quant;      
    diff_quant_noise=mxm*sign(diff_quant_noise).*(abs(diff_quant_noise)/Q).^(1/P);
    OUTIMG(:,y)=round(OUTIMG(:,y-1)+diff_quant_noise);
	image(OUTIMG);axis equal;axis off;
	title(['Output image, mxm=',num2str(mxm),'; BPP=',num2str(BPP),'; P=',num2str(P)],'FontWeight','Bold');
	drawnow
end
[mn,stderr]=std_2d(INPIMG-OUTIMG);
image(OUTIMG);axis equal;axis off
title(['Output image, mxm=',num2str(mxm),'; BPP=',num2str(BPP),'; P=',num2str(P)],'FontWeight','Bold');
figure(3);
[mn,stdev]=std_2d(INPIMG-OUTIMG);
disp_norm(INPIMG-OUTIMG,4);
title(['Reconstruction error; ErrStDev=',num2str(stdev)],'FontWeight','Bold')
%---------------------------------------------------------------------------
function OUTIMG=dct_codng(INPIMG,SzW,SzM,Q,P)
% Test DCT block-wise image coding
% Q-number of bits per coefficient in nonuniform coding 
% P-th low nonlinearity for nonuniform coding
% Szw - size of the square block
% SzM - horizontal size of coding mask (<SzW);
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Call OUTIMG=dctcodng(INPIMG,SzW,SzM,Q,P)

fig_n(4);
figure(1);
display_1(INPIMG);
title('DCT image coding: Input image');drawnow
[SzX SzY]=size(INPIMG);
SzXX=fix(SzX/8);SzYY=fix(SzY/8);
figure(2);
mask=ones(8);mask(1,:)=0.95;mask(8,:)=0.95;mask(:,1)=0.95;mask(:,8)=0.95;
mask=kron(ones(SzXX,SzYY),mask);

[SzMX,SzMY]=size(mask);
INPIMG=INPIMG(1:SzMX,1:SzMY);
display_1(INPIMG.*mask);
title('Image 8x8 blocks');
drawnow
% Generation of the coding mask
mask=zeros(SzW);
for x=1:SzW, 
	for y=1:SzM-x+1,
		mask(x,y)=1;
	end
end
Cf=(Q+1)*(sum(sum(mask)))/(SzW*SzW);
Q=2^Q;
OUTIMG=INPIMG;
for x=1:SzX/SzW,
	for y=1:SzY/SzW,
		fragment=INPIMG((x-1)*SzW+1:x*SzW,(y-1)*SzW+1:y*SzW);
		frgmsp=dct2(fragment);
% Quantization of spectra
		s=sign(frgmsp); abs_frgmsp=abs(frgmsp);
		mx=max(max(frgmsp));
		abs_frgmsp=mx*((round(Q*(abs_frgmsp/mx).^P)/Q).^(1/P));
		frgmsp=s.*mask.*abs_frgmsp;
		OUTIMG((x-1)*SzW+1:x*SzW,(y-1)*SzW+1:y*SzW)=idct2(frgmsp);
	end
   figure(3);
   display_1(OUTIMG);
	title(['Reconstructed image, SzW=',num2str(SzW),'; SzMask=',num2str(SzM),'; Q=',num2str(Q),'; P=',num2str(P)]);drawnow
   figure(4);
   disp_norm(OUTIMG-INPIMG,6);
   title('Reconstruction error');drawnow
end

[mn,err]=std_2d(OUTIMG-INPIMG);
figure(4);
disp_norm(OUTIMG-INPIMG,6);
title(['Reconstraction error; std=',num2str(err),'; BPP=',num2str(Cf)]);

%--------------------------------------------------------------------------
function OUT=img_histo(INPIMG);

% Image histogram
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUT=img_histo(INPIMG);

[SzX SzY]=size(INPIMG);

histbuf=zeros(256,SzY);
for n=1:SzX
buf=INPIMG(n,:); buf=buf+256*([0:SzY-1]);
histbuf(buf+1)=histbuf(buf+1)+ones(1,SzY);
end

OUT=sum(histbuf'); 
