function diffrentiat_integrat_error_CRC;

% Comparison diffrentiation/integration error for DCT based diffrentiation
% and d2&Simpson differentiation&integration by means of Nit times of a 1D signal
% iteration of differentiation and integration operation
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il),

Nit=inputdlg({'The number of iterations Nit='},'Enter interation number',1,{'100'});
Nit=str2double(Nit);

INPUT=zeros(256,1);INPUT(105-5:105+5)=1;
SzX=length(INPUT);
X=1:SzX;
x=round(SzX/8):7*round(SzX/8);
mn=min(INPUT);
INPUT=(INPUT-min(INPUT))/(max(INPUT)-min(INPUT));
OUT_DCT=zeros(1,Nit);
OUT_D2=zeros(1,Nit);
OUT_dct=INPUT;
OUT_d2=INPUT;
figure_n(4);
for n=1:Nit,
    OUT_dct=differentiator1D_DCT(OUT_dct);
    OUT_dct=integrator1D_DCT(OUT_dct);
    OUT_dct_mn=OUT_dct(x)-min(OUT_dct(x));
    [mn,STD_DCT]=std2d(INPUT(x)-OUT_dct_mn);
    OUT_DCT(n)=STD_DCT;
    figure(1)
    plot(x,INPUT(x),'b',x,OUT_dct_mn,'r.','LineWidth',2);
    title(['DCT Diff/Integr: ',num2str(Nit),' iterations'],'FontWeight','Bold','FontSize',12)
    xlabel('Sample index','FontWeight','Bold','FontSize',12)
    axis([round(SzX/8) 7*round(SzX/8) 0 1.1]);
    grid on
    legend('Initial signal','Iterated dif-integr signal')
    set(gca,'FontWeight','Bold','FontSize',12);
    
    figure(2)
    plot(1:Nit,OUT_DCT,'b','LineWidth',2);
    title(['DCT Diff/Integr: ErrorStDev=',num2str(STD_DCT,3)],'FontWeight','Bold','FontSize',12)
    xlabel('Iterations','FontWeight','Bold','FontSize',12)
    axis tight;
    grid on
    set(gca,'FontWeight','Bold','FontSize',12);
    drawnow
    OUT_d2=differentiator1D_d2(OUT_d2);
    OUT_d2=integrator_trap(OUT_d2);
%     OUT_d2=[OUT_d2(2:SzX);OUT_d2(1)];
    OUT_d2_mn=OUT_d2(x)-min(OUT_d2(x));
    [mn,STD_D2]=std2d(INPUT(x)-OUT_d2_mn);
    OUT_D2(n)=STD_D2;
    figure(3)
    plot(x,INPUT(x),'b',x,OUT_d2_mn,'r.','LineWidth',2);
    axis([round(SzX/8) 7*round(SzX/8) 0 1.1]);
    title(['D2/Trap Differ/Integr:',num2str(Nit),' iterations'],'FontWeight','Bold','FontSize',12)
    xlabel('Sample index','FontWeight','Bold','FontSize',12)
    grid on
    legend('Initial signal','Iterated dif-integr signal')
    set(gca,'FontWeight','Bold','FontSize',12);
    figure(4)
    plot(1:Nit,OUT_D2,'b','LineWidth',2);
    axis tight
    title(['D2/Trap Diff/Integr:ErrorStDev=',num2str(STD_D2,3)],'FontWeight','Bold','FontSize',12);
    xlabel('Iterations','FontWeight','Bold','FontSize',12)
    grid on
    set(gca,'FontWeight','Bold','FontSize',12);
    drawnow
end

%---------------------------------------------------------
function OUT=integrator_trap(INPUT);

% Trapezoidal integrator  
% 
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Usage : OUT=integrator_simpson(INPUT);

SzX=max(size(INPUT));
INPUT=reshape(INPUT,SzX,1);
INPUT=[INPUT(1);INPUT;INPUT(SzX)];
OUT=INPUT;
N=length(INPUT);
k=1:N-1;
for n=1:SzX
    OUT(n+1)=OUT(n)+(INPUT(n+1)+INPUT(n))/2;
end
OUT=OUT(2:SzX+1);
%-------------------------------------------------

function OUT=integrator_simps(INPUT);
% Simpson- integrator 
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Usage : OUT=integrator_simpson(INPUT);

SzX=max(size(INPUT));
INPUT=reshape(INPUT,SzX,1);
INPUT=[INPUT(2);Input(1);INPUT;INPUT(SzX);INPUT(SzX-1)];
OUT=INPUT;
N=length(INPUT);
k=1:N-1;
for n=1:SzX
    OUT(n+1)=OUT(n)+(INPUT(n+1)+INPUT(n))/2;
end

%----------------------------------------------------------------------
function OUT=differentiator1D_DCT(INPUT);

% DCT domain implementation of differentiating ramp-filter
% OUT(k)=(-1)^k*IDCT((N-r).*DCT(N-r)(INPUT))
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il), Kfar Saba, June 9, 2010
% Usage: OUT=differentiator1D_DCT(INPUT);

[SzX,SzY]=size(INPUT);
INPUT=reshape(INPUT,max(SzX,SzY),min(SzX,SzY));
N=max(SzX,SzY);
k=0:N-1;
sp=dct(INPUT);
sp_reshaffled=[0;flipud(sp(2:N))];
OUT=-(pi)*((-1).^k)'.*idct((N-k)'.*sp_reshaffled)/N;
%-------------------------------------------------------------------

function OUT=integrator1D_DCT(INPUT);

% DCT domain implementation of differentiating ramp-filter
% OUT(k)=(-1)^k*IDCT((N-r).*DCT(N-r)(INPUT))
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il), Kfar Saba, June 9, 2010
% Usage: OUT=integrator1D_DCT(INPUT);

[SzX,SzY]=size(INPUT);
INPUT=reshape(INPUT,max(SzX,SzY),min(SzX,SzY));
N=max(SzX,SzY);
k=0:N-1;
sp=dct(INPUT);
sp_reshaffled=[0;flipud(sp(2:N,:))];
OUT=N*((-1).^k)'.*idct((1./(N-k)').*sp_reshaffled)/(pi)+kron(mean(INPUT),k'-2*mean(k));

%---------------------------------------------------------------------------
function figure_n(N)
% Generating N square figure windows of ScreenVerSz/2*ScreenVerSz pixels
% figure('Position',[HorPosition VerPosition HorSize VertSize]);
% Coordinate origin - left bottom corner of the screen
% Version July 15, 2001; Modified Oct. 2008, Tampere
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call figure_n(N)

% Generating a figure window
ScrSz=get(0,'ScreenSize');
HorSz=(ScrSz(3)-20); VerSz=(ScrSz(4));
% close all;

switch N;
case 1, close all
   Nvert=1;Nhor=1;H_Sz=(HorSz-80); V_Sz=(VerSz-120);
%    figure('Position',[30 VerSz-30-V_Sz V_Sz V_Sz]); colormap(gray(256))
    figure('Position',[80 VerSz-80-V_Sz V_Sz V_Sz]); colormap(gray(256))

case 2, close all 
   Nvert=1;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-200);
%    figure('Position',[30 VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
    figure('Position',[30 VerSz-200-H_Sz H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30+H_Sz VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
   figure('Position',[80+H_Sz VerSz-200-H_Sz H_Sz V_Sz]);colormap(gray(256))

case 3,close all 
%    Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
    Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-220)/2;
%    figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[80+H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))

case 4,close all; 
%    Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-220)/2;
%    figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[80+H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[80+H_Sz VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))
case 5,
   close all
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-220)/2;
   figure('Position',[30 VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[60+H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[85+2*H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[60+H_Sz VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))
case 6, 
   close all
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-220)/2;
   figure('Position',[30 VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[60+H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[85+2*H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[60+H_Sz VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[80+2*H_Sz VerSz-180-2*V_Sz H_Sz V_Sz]);colormap(gray(256))
end

%---------------------------------------------------------
%--------------------------------------------------------------------------
function [mn,v]=std2d(INP);

% Standard deviation and mean of a 2-D array
% z=[mean stdeviation];
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call [mn v]=std_2d(INP)

[SzX SzY]=size(INP);
mn=sum(sum(INP))/(SzX*SzY);
v=sqrt(sum(sum((INP-mn).^2))/(SzX*SzY));
%-------------------------------------------------------------------------
%-------------------------------------------------------------------------
function OUT=differentiator1D_d2(INPUT);

% Numerical differentiating d2-filter
% 
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il), Kfar Saba, June 9, 2010
% Usage: OUT=differentiator1D_d2(INPUT);

[SzX,SzY]=size(INPUT);
INPUT=reshape(INPUT,max(SzX,SzY),min(SzX,SzY));
N=max(SzX,SzY);
k=0:N-1;

d2=[-1/12, 2/3,0,-2/3,1/12]';
OUT=conv2(INPUT,d2,'same');
%--------------------------------------------------------------------------