function Histo_standardization_CRC;

% Standardization of image histogram; 
% Histogram of INPIMG1 is made identical to histogram of INPIMG2
% 
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  


clf
load interfero_distort;
INPIMG1=interfero_distort;
load interfero_templ;
INPIMG2=interfero_templ;
figure_n(4);
figure(1);
display1(INPIMG1);title('Input image','FontWeight','Bold');
figure(2);
display1(INPIMG2);title('Reference image','FontWeight','Bold');
[SzX1 SzY1]=size(INPIMG1);
[SzX2 SzY2]=size(INPIMG2);

histbuf=zeros(256,SzY1);
for n=1:SzX1
buf=INPIMG1(n,:); 
buf=buf+256*([0:SzY1-1]);
histbuf(buf+1)=histbuf(buf+1)+ones(1,SzY1);
end
h1=sum(histbuf');

cumh=cumsum(h1);
lut1=255*cumh/max(cumh);
histbuf=zeros(256,SzY2);
for n=1:SzX2
buf=INPIMG2(n,:); 
buf=buf+256*([0:SzY2-1]);
histbuf(buf+1)=histbuf(buf+1)+ones(1,SzY2);
end
h2=sum(histbuf');

figure(3)
plot(h1/(SzX1*SzY1),'k--','LineWidth',2);
hold on
plot(h2/(SzX2*SzY2),'k','LineWidth',2);
axis tight; grid on
title('Input and reference image histograms','FontWeight','Bold');
set(gca,'FontWeight','Bold');
drawnow
legend('Inp.image histogram','Ref. image histogram');
hold off 

cumh=cumsum(h2);lut2=255*cumh/max(cumh);

for q=1:256,
   lut(q)=sum(lut2<=lut1(q));
end
lut=lut.*(lut>0);lut=lut.*(lut<=255)+255*(lut>255);
OUTIMG=reshape(lut(1+INPIMG1),SzX1,SzY1);

figure(4);
display1(OUTIMG);title('Output image','FontWeight','Bold');

histbuf=zeros(256,SzY1);
for n=1:SzX1
buf=OUTIMG(n,:); buf=buf+256*([0:SzY1-1]);
histbuf(buf+1)=histbuf(buf+1)+ones(1,SzY1);
end
%-----------------------------------------------------------
function figure_n(N)
% Generating N square figure windows of ScreenVerSz/2*ScreenVerSz pixels
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Version July 15, 2001
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call figure_n(N)

% Generating a figure window
ScrSz=get(0,'ScreenSize');
HorSz=(ScrSz(3)-20); VerSz=(ScrSz(4));
close all;

switch N;
case 1, 
   Nvert=1;Nhor=1;H_Sz=(HorSz-80); V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-V_Sz V_Sz V_Sz]); colormap(gray(256))
case 2, 
   Nvert=1;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
case 3, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 4, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 5,
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 6, 
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-75-2*V_Sz H_Sz V_Sz]);colormap(gray(256))
end
%--------------------------------------------------------------------
function display1(INPIMG)
% Displays a 2-D signal as an image 
% with normalization by gobal maximum and minimum
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Call display1(INPIMG)

clf
colormap(gray(256))
Min=min(min(INPIMG)); Max=max(max(INPIMG));
if Max-Min==0, im=zeros(size(INPIMG)); else im=255*(INPIMG-Min)/(Max-Min); end

image(im);axis image;axis off;
